----------------------------------------------
-- renderBox.lua
-- Author - Vincent Giangiulio a.k.a Vincent
-- Edit by Lygin Gennady a.k.a Croc 13.01.2017
-- Integration by Lygin Gennady a.k.a Croc 18.03.2017
-- Edit by Patrick Ziegler a.k.a. Gironde 06.04.2017
----------------------------------------------
require "locale";
require "closeButton";
----------------------------------------------
-- Define local functions
----------------------------------------------

local setLabelElementName = function(str)
    if str == "Constell. in Latin" then
        strl = "i18nconstellations";
    else
       strl = string.lower(string.gsub(str, " ", ""));
    end
   return strl;
end

local setRenderElementName = function(str)
    strr = string.lower(string.gsub(str, " ", ""));
    return strr;
end

local setOrbitElementName = function(str)
   strr = string.gsub(str, " ", "");
   return strr;
end

local setCheckBoxText = function(box, k, flag) -- Function used to query the render table displayed and to check the box
   if flag then
        box[k].Text = "v";
    else
        box[k].Text = "";
    end
end
--===================================
local makeClassBox = function(classBox, k, v, leftOffset) -- Function used to create a box for each render
    classBox = CXBox:new()
        :init(0, 0, 0, 0)
        :textfont(normalfont)
        :textcolor(cbutextoff)
        :movetext(0, 8)
        :text(_(v))         
        :movable(false)
        :attach(renderFrame, leftOffset+15, renderFrameHeight-25-(20*k), renderFrameWidth-leftOffset-100, 14+(20*k))
    end
-- ===================================
local makeCheckBox = function(checkBox, k, leftOffset) -- Function used to create a check box for each render
    checkBox[k] = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(checkBcolor)
        :fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(cbutextoff)
        :textpos("center")
        :movetext(20, 10)
        :text("")
        :movable(false)
        :active(true)
        :attach(renderFrame, leftOffset, renderFrameHeight-25-(20*k), renderFrameWidth-leftOffset-11, 14+(20*k))
   end

------------------------------------------------------------
---- Set initial values render frame -----
------------------------------------------------------------
local nrender = table.getn(renderclass);
local nguide = table.getn(guideclass);
local norbit = table.getn(orbitclass);
local nlabel = table.getn(labelclass);
local nmax = math.max(nrender, nguide + 2, norbit, nlabel);

local newwidth, newheight;
renderFrameWidth, renderFrameHeight = 380, (20 * nmax)+20;  -- размеры панели ширина/высота 

renderLeftOffset = 10;    
orbitLeftOffset = 30;      
labelLeftOffset = 50;     
guideLeftOffset = 250-30;  

starStyleLeftOffset = guideLeftOffset - 2;
starStyleUpOffset = 9 + 20 * (nguide + 2);
textureResLeftOffset = guideLeftOffset - 2;
textureResUpOffset = 9 + 20 * (nguide + 3);

renderCheckBox = {};
guideCheckBox = {};
orbitCheckBox = {};
labelCheckBox = {};

rendertable = {};
orbittable = {};
labeltable = {};

----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------

-- command's button --------------------------------------------------------
renderBox = CXBox:new() -- renderBox contains the 'set renders' and 'set adds' buttons
    :init(0, 0, 0, 0)
    :movable(false)
    :attach(screenBox, width, height, 0, 0) 

--[[  
setRenderButton = CXBox:new() -- frame of 'set renders' button
    :init(0, 0, 0, 0)
    :bordercolor({0.2,0.8,1,.6})
    :fillcolor(cFrameFill)
    :textfont(normalfont)
    :textcolor({0.2,0.8,1,1})
    :textpos("left")
    :movetext(7, 8)
    :text(_("Set Render"))
    :movable(false)
    :active(true) 
    :attach(menuLine2Box, -91, 0, 96, 0) -- :attach(menuLine2Box, -88+1-4, 0, 100-4, 0)

setRenderButton.Action = (function()
    return
        function()
            renderFrame.Visible = not(renderFrame.Visible);
                    if renderFrame.Visible then
                        renderFrame:orderfront();
                        setRenderButton.Bordercolor = {1,0,0,.6}
                    else
                        setRenderButton.Bordercolor = {0.2,0.8,1,.6}
                    end
       end
  end) ()
]]
-- frame ------------------------------------------------ 
renderFrame = CXBox:new() -- frame of renders in screenBox
    :init(0, 0, 0, 0)
    :fillcolor(csetbofill)
    :bordercolor(cbubordoff)   
    :movable(true)
    :visible(false)
    :attach(screenBox, (width-renderFrameWidth)/2+100, height-renderFrameHeight-2, (width-renderFrameWidth)/2-100, 2);  

renderFrame.Customdraw =  -- The customdraw functions rotate continuously when the frame is created
    function(this)
        -- We start displaying the text on the second tick in order to
        -- hide a possible uggly resizing of the box during the first tick.
        if render_firstTickDone then
            textlayout:setfont(normalfont);

            textlayout:setfontcolor({0.2,0.8,1,0.7});
            textlayout:setpos(this.lb+renderLeftOffset+190+20, this.tb-185-20);     -- Change title : "Show"     
            textlayout:println(_("Show:"));

            textlayout:setpos(this.lb+guideLeftOffset, this.tb-20);                         -- Change title : "Guides"  
            textlayout:println(_("Guides:"));

            textlayout:setpos(this.lb+10, this.tb-20);                                           -- Change title : "Objects / Orbits / Labels:"    
            textlayout:println(_("Objects / Orbits / Labels:"));

            textlayout:setpos(this.lb+guideLeftOffset, this.tb-84);                          -- Change title : "Grid"
            -- textlayout:println(_("Grid:"));   

         --   textlayout:setfontcolor({1,1,1,0.7});
            textlayout:setpos(this.lb+70, this.tb-185);                                           -- Change title : "Planets and other objects:"
            textlayout:println(_("Planets and other objects:"));

            if not(renderCloseButtonDone) then
                renderCloseButtonDone = true;
                makeCloseButton(renderCloseButton, renderFrame, setRenderButton);
            end
        else
            render_firstTickDone = true;
        end

        if newwidth ~= width or newheight ~= height then
            newwidth, newheight = width, height;          
            renderFrame:attach(screenBox, width-renderFrameWidth-205, height-renderFrameHeight-10, 205, 10);        
        end  
end;
-- ================= installation and check  of renderflags render=================== 1st column boxes
for k, v in ipairs(renderclass) do

    makeClassBox(renderClassBox, k, "", renderLeftOffset); -- create an new element
    makeCheckBox(renderCheckBox, k, renderLeftOffset); -- -- create an new check-box

    if v == "" then
        renderCheckBox[k].Visible = false;
    end

    local vv, kk = v, k;

    renderCheckBox[k].Customdraw =
        function(this)
            vr = setRenderElementName(vv);
            renderFlag = celestia:getrenderflags()[vr];
            setCheckBoxText(renderCheckBox, kk, renderFlag) -- checked or not checked
        end;

    renderCheckBox[k].Action = (function()
        return
            function()
                vr = setRenderElementName(vv);
                renderFlag = celestia:getrenderflags()[vr];
                renderFlag = not(renderFlag);
                rendertable[vr] = renderFlag;
                celestia:setrenderflags(rendertable);
            end
        end) ();
end

-- ==================  installation and check  of renderflags orbit ================ 2nd column boxes
for k, v in ipairs(orbitclass) do

    makeClassBox(orbitClassBox, k, "", orbitLeftOffset);
    makeCheckBox(orbitCheckBox, k, orbitLeftOffset);

    if v == "" then
        orbitCheckBox[k].Visible = false;
    end

    local vv, kk = v, k;

    orbitCheckBox[k].Customdraw =
        function(this)
            vo = setOrbitElementName(vv);
            orbitFlag = celestia:getorbitflags()[vo];
            setCheckBoxText(orbitCheckBox, kk, orbitFlag); -- appel function line 32
        end;

    orbitCheckBox[k].Action = (function()
        return
            function()
                vo = setOrbitElementName(vv);
                orbitFlag = celestia:getorbitflags()[vo];
                orbitFlag = not(orbitFlag);
                orbittable[vo] = orbitFlag;
                celestia:setorbitflags(orbittable);
            end
        end) ();

 end
-- =================  installation and check  of renderflags guide ================ 3rd column of boxes
for k, v in ipairs(guideclass) do

    makeClassBox(guideClassBox, k, v, guideLeftOffset);
    makeCheckBox(guideCheckBox, k, guideLeftOffset);

    if v == "" then
        guideCheckBox[k].Visible = false;
    end

    local vv, kk = v, k;

    guideCheckBox[k].Customdraw =       -- Constantly observes the state of the flags in celestia : interaction with celestia
        function(this)
            vr = setRenderElementName(vv);
            renderFlag = celestia:getrenderflags()[vr];
            setCheckBoxText(guideCheckBox, kk, renderFlag) -- Call function : check or not check
        end;

    guideCheckBox[k].Action = (function() -- change the check and Celestia's update
        return
            function()
                vr = setRenderElementName(vv);
                renderFlag = celestia:getrenderflags()[vr];
                renderFlag = not(renderFlag);
                rendertable[vr] = renderFlag;
                celestia:setrenderflags(rendertable);
            end
        end) ();
end
-- ================  installation and check of renderflags label (name) ================ 4th column boxes
for k, v in ipairs(labelclass) do

    makeClassBox(labelClassBox, k, v, labelLeftOffset)
    makeCheckBox(labelCheckBox, k, labelLeftOffset)

    if v == "" then
        labelCheckBox[k].Visible = false;
    end

    local vv, kk = v, k;

    labelCheckBox[k].Customdraw =
        function(this)
            vl = setLabelElementName(vv);
            labelFlag = celestia:getlabelflags()[vl];
            if vv == "Constell. in Latin" then
                setCheckBoxText(labelCheckBox, kk, not(labelFlag)) -- appel function line 32
            else
                setCheckBoxText(labelCheckBox, kk, labelFlag) -- appel function line 32
            end
        end;

    labelCheckBox[k].Action = (function()
        return
            function()
                vl = setLabelElementName(vv);
                labelFlag = celestia:getlabelflags()[vl];
                labelFlag = not(labelFlag);
                labeltable[vl] = labelFlag;
                celestia:setlabelflags(labeltable);
            end
     end) ();
end 

--=========================================
-- stardistance box and frame : container renderFrame
-- Original Author of algorythme - Vincent Giangiulio a.k.a Vincent
-- Adapted by Patrick Ziegler alias Gironde 27-03-2017
-- Edit by Lygin Gennady a.k.a  12.04.2017
--=========================================

-------------------------------- star distance frame ---------------------------------------------------------

-- There is no method to retrieve or change the cursor position in Celestia stardistance window
-- Only the number size

----------------------------------------
-- set initial values
----------------------------------------

--local checkBcolor = color2 -- color2 or cbubordoff

local stardistanceLimitFrameWidth, stardistanceLimitFrameHeight = 200, 43   -- dimensions of frame of complete scrollbar
local stardistanceLimitTrackFrameW = stardistanceLimitFrameWidth-16         
local stardistanceLimitCursorframeW = 12  		                                    -- width of cursor of stardistance
local stardistanceLimitCursorMargin = 3  	                                            -- margin of cursor
local addLeftOffset = 10+5; -- edit Croc
local stardistanceLimit = celestia:getstardistancelimit()
local laststardistanceLimit = nil                                                                 -- Buffer variable
local stardistanceLimit2 = nil                                                                     -- Buffer variable
local laststardistanceLimit2 = 1000000                                                       -- Buffer variable
local stardistLimScale = 1000000                                                               -- scale of track and cursor
local changeScaleCaseCheck = false                                                           -- flag : one of casecheck have been used or not

local stardistanceLimitPosMax = stardistanceLimitTrackFrameW - stardistanceLimitCursorframeW - (2 * stardistanceLimitCursorMargin); -- position max of cursor on the track,  edit Jogad
local stardistanceLimitPos = math.ceil(celestia:getstardistancelimit() * stardistanceLimitPosMax / stardistLimScale); 	-- convert Ly in pixel the position of cursor edit Jogad

-----------------------------------------------
-- set up and draw
-----------------------------------------------
stardistanceLimitFrame = CXBox:new() -- frame for label and track : prinicipal frame of adjusting tool
    :init(0, 0, 0, 0)
    --:bordercolor({1, 1, 0, 1})
    --:fillcolor({1,1,0,.2})
    :movable(false)
    :attach(renderFrame, addLeftOffset-7, 47, 
                renderFrameWidth-stardistanceLimitFrameWidth-addLeftOffset+7, renderFrameHeight-stardistanceLimitFrameHeight-25-20-2);

stardistanceLimitFrame.Customdraw =
    function(this)

        stardistanceLimit = celestia:getstardistancelimit() -- update and set display of LUT
    

        textlayout:setfont(normalfont)
        textlayout:setpos(this.lb+7+3, this.tb-15)
        textlayout:setfontcolor({0.2,0.8,1,0.7}) 
        textlayout:println(_("Limit Star Distance (ly):"))

        stardistanceLimit_str = string.format("%1.0f", math.floor(stardistanceLimit))
        textlayout:setpos(this.lb+142, this.tb-15)        
        textlayout:setfontcolor(cbutextoff)         
        textlayout:println(stardistanceLimit_str)    

-- ********* 1)  changing on track and cursor by changing scale of track ***********

        -- The scale has been changed (stardistLimScale)
        -- see firsttickdone
        if changeScaleCaseCheck == true then -- use only after FirstTickDone
            changeScaleCaseCheck = false -- reset
            --stardistanceLimit = stardistLimScale -- Force stardistanceLimit to the high value of the scale
            
            stardistanceLimitPos = math.floor(stardistanceLimit * stardistanceLimitPosMax / stardistLimScale) -- convert in pixel -- math.ceil = math_round              
                -- update display

            stardistanceLimitCursorFrame.Customdraw ()
            celestia:setstardistancelimit(stardistanceLimit) -- set celestia           
            -- The rest is done by the same code as below           
        end

-- ********* 2) search position of cusor on the screen **************************
            
        -- Determines the stardistancelimit for celestia, according to the position of the cursor on the track.
        -- Used when we move the cursor.
        stardistanceLimit_lb = stardistanceLimitTrackFrame.lb; -- Indicates a position of the left edge relative to the right edge of the screenbox
        stardistanceLimit_lb2 = stardistanceLimitCursorFrame.lb;-- Indicates a position of the left edge relative to the right edge of the screenbox
        
        -- for define the value
        -- the distance is calculated by difference between lb2 et lb (pixel) and converted this in Ly
        -- calcul of number of Ly       
        stardistanceLimit2 =  math.ceil(stardistanceLimit_lb2 - stardistanceLimit_lb - stardistanceLimitCursorMargin) * stardistLimScale / stardistanceLimitPosMax; -- convert  in Ly -- round in celestia

        if stardistanceLimit2 < 1 then -- 1 is the bottom limit
            stardistanceLimit2 = 1;
        end
        
        -- -- laststardistanceLimit = nil or stardistanceLimit before movement
        if laststardistanceLimit2 ~= stardistanceLimit2 and not(stardistanceLimitTrack_FirstTickDone) then
            -- The cursor has been moved; set the new celestia stardistanceLimit with stardistanceLimit2   and not stardistanceLimit ( update at the beginning of this customdraw)         
            laststardistanceLimit2 = stardistanceLimit2 -- laststardistanceLimit2 take the new value
            celestia:setstardistancelimit(laststardistanceLimit2) -- set celestia
        end

        stardistanceLimitTrack_FirstTickDone = false; -- stardistanceLimitTrack_FirstTickDone = false;

-- ********* 3) Cursor operation *******************************************

        stardistanceLimit = celestia:getstardistancelimit() -- update stardistanceLimit in LUT or stardistanceLimit = laststardistanceLimit2

        --  test of scale received from celestia for changing automatically the scale of LUT

        if stardistanceLimit > 100000 then
            Case5CheckFrame.Action ()
        elseif stardistanceLimit <= 100000 and stardistanceLimit > 10000 then  
            Case4CheckFrame.Action ()
        elseif stardistanceLimit <= 10000 and stardistanceLimit > 1000 then  
            Case3CheckFrame.Action ()
        elseif stardistanceLimit <= 1000 and stardistanceLimit > 100 then  
            Case2CheckFrame.Action () 
        elseif stardistanceLimit <= 100 then 
            Case1CheckFrame.Action () 
        end 
        
        --------------------------------------------------------------------------------   
        if laststardistanceLimit ~= stardistanceLimit then
                -- cursor has moved, set the new position of the cursor.
            laststardistanceLimit = stardistanceLimit
                -- to define the position use the stardistLimScale calculation in LUT, not 1000000     
            stardistanceLimitPos = math.floor(stardistanceLimit * stardistanceLimitPosMax / stardistLimScale) -- convert in pixel -- math.ceil = math_round              
                -- update display
            stardistanceLimitCursorFrame:attach(stardistanceLimitTrackFrame, stardistanceLimitCursorMargin+stardistanceLimitPos, 0, stardistanceLimitTrackFrameW-stardistanceLimitCursorMargin-stardistanceLimitPos-stardistanceLimitCursorframeW, 0)
         end
         
     end;
--****************************** End of Customdraw function *************************

--------------------------------------------------------------------------------------------------------------------
----------------------------      child frames    ------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------
           
------------------------------- star distance track of adjusting tool -----------------------
stardistanceLimitTrackFrame = CXBox:new()
    --:bordercolor(cslidefrbord)
    --:fillcolor({.2,.2,.6,.1})
    :visible(true)    
    :attach(stardistanceLimitFrame, 8, 8, 8, 24)

----------------------------------- star distance cursor of adjusting tool ---------------------

stardistanceLimitCursorBox = CXBox:new()
    :bordercolor(cslidefrbord)
    --:fillcolor({.2,.2,.6,.1})
    :visible(true)
    :attach(stardistanceLimitTrackFrame, 2, 4, 2, 4)

stardistanceLimitCursorFrame = CXBox:new()
    :bordercolor(checkBcolor)  --cslidebobord
    :fillcolor(cslidebofill)
    :visible(true)
    :movable(true)
    :attach(stardistanceLimitTrackFrame, stardistanceLimitCursorMargin+stardistanceLimitPos, 0, 
                stardistanceLimitTrackFrameW-stardistanceLimitCursorMargin-stardistanceLimitPos-stardistanceLimitCursorframeW, 0);

stardistanceLimitCursorBox.Masked = stardistanceLimitCursorFrame;

stardistanceLimitCursorFrame.Customdraw =           
    function(this)
        -- redraw the cursor frame    
        stardistanceLimitCursorFrame:attach(stardistanceLimitTrackFrame, stardistanceLimitCursorMargin+stardistanceLimitPos, 0, stardistanceLimitTrackFrameW-stardistanceLimitCursorMargin-stardistanceLimitPos-stardistanceLimitCursorframeW, 0);       
    end;

-- ******************************************************************
-- add check cases stardistance in renderFrame
-- ******************************************************************
-- stardistanceLimitFrame

Case1LabelFrame = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(smallfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(-4, 9) 
    :text("100")        
    :movable(false)
    :active(true) 
    :attach(stardistanceLimitFrame, 23, -12, 172, 45)

Case1CheckFrame = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(checkBcolor)
        --:fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(color0)
        :textpos("left")
        :movetext(-3, 12)
        :text("")
        :movable(false)
        :active(true)
        :attach(stardistanceLimitFrame, 12, -8, 179, 47)

Case1CheckFrame.Action = (function()                        -- modify the check case and stardistLimScale
        return
            function()
          
                if stardistLimScale ~= 100 then  
                    Case1CheckFrame.Text = "o"
                    Case2CheckFrame.Text = ""
                    Case3CheckFrame.Text = ""
                    Case4CheckFrame.Text = "" 
                    Case5CheckFrame.Text = "" 

                    changeScaleCaseCheck  = true                    
					stardistLimScale = 100   
                end              
            end
        end) ();

Case2LabelFrame = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(smallfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(-4, 9) 
    :text("1K")  
    :movable(false)
    :active(true) 
    :attach(stardistanceLimitFrame, 23+38, -12, 172-38, 45)

Case2CheckFrame = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(checkBcolor)
        --:fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(color0)
        :textpos("left")
        :movetext(-3, 12)
        :text("")
        :movable(false)
        :active(true)
        :attach(stardistanceLimitFrame, 12+38, -8, 179-38, 47)
   
Case2CheckFrame.Action = (function()                        -- modify the check case and stardistLimScale
        return
            function()
          
                if stardistLimScale ~= 1000 then 
                    Case1CheckFrame.Text = ""
                    Case2CheckFrame.Text = "o"
                    Case3CheckFrame.Text = ""
                    Case4CheckFrame.Text = "" 
                    Case5CheckFrame.Text = "" 

                    changeScaleCaseCheck  = true                    
					stardistLimScale = 1000   
                end              
            end
        end) ();

Case3LabelFrame = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(smallfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(-4, 9) 
    :text("10K")  
    :movable(false)
    :active(true) 
    :attach(stardistanceLimitFrame, 23+38*2, -12, 172-38*2, 45)

Case3CheckFrame = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(checkBcolor)
        --:fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(color0)
        :textpos("left")
        :movetext(-3, 12)
        :text("")
        :movable(false)
        :active(true)
        :attach(stardistanceLimitFrame, 12+38*2, -8, 179-38*2, 47)

Case3CheckFrame.Action = (function()                        -- modify the check case and stardistLimScale
        return
            function()
                if stardistLimScale ~= 10000 then    
                    Case1CheckFrame.Text = ""
                    Case2CheckFrame.Text = ""
                    Case3CheckFrame.Text = "o"
                    Case4CheckFrame.Text = ""
                    Case5CheckFrame.Text = ""
                    
                    changeScaleCaseCheck  = true                     
					stardistLimScale = 10000   
                end
            end
        end) ();
      
Case4LabelFrame = CXBox:new()
    :init(0, 0, 0, 0)
    --:bordercolor(cbubordoff)      
    --:fillcolor(color21)
    :textfont(smallfont) 
    :textcolor(color1) -- color9
    :textpos("left")    
    :movetext(-4, 9) 
    :text("100K")    
    :movable(false)
    :active(true) 
    :attach(stardistanceLimitFrame, 23+38*3, -12, 172-38*3, 45)

Case4CheckFrame = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(checkBcolor)
        --:fillcolor({1,1,0,.2})
        :textfont(smallfont)
        :textcolor(color0)
        :textpos("left")
        :movetext(-3, 12)
        :text("")
        :movable(false)
        :active(true)
        :attach(stardistanceLimitFrame, 12+38*3, -8, 179-38*3, 47)

Case4CheckFrame.Action = (function()                        -- modify the check case and stardistLimScale
        return
            function()
                if stardistLimScale ~= 100000 then 
                    Case1CheckFrame.Text = ""
                    Case2CheckFrame.Text = ""
                    Case3CheckFrame.Text = ""
                    Case4CheckFrame.Text = "o"
                    Case5CheckFrame.Text = ""
                    
                    changeScaleCaseCheck  = true                    
					stardistLimScale = 100000
                end
            end
        end) ();
      
Case5LabelFrame = CXBox:new()
        :init(0, 0, 0, 0)
        --:bordercolor(cbubordoff)      
        --:fillcolor(color21)
        :textfont(smallfont) 
        :textcolor(color1) -- color9
        :textpos("left")    
        :movetext(-4, 9) 
        :text("1M")  
        :movable(false)
        :active(true) 
        :attach(stardistanceLimitFrame, 23+38*4, -12, 172-38*4, 45)

Case5CheckFrame = CXBox:new()
        :init(0, 0, 0, 0)
        :bordercolor(checkBcolor)
        :textfont(smallfont)
        :textcolor(color0) -- cbutextoff
        :textpos("left") 
        :movetext(-3, 12)
        :text("o") -- for run the program, after modified by check
        :movable(false)
        :active(true)
        :attach(stardistanceLimitFrame, 12+38*4, -8, 179-38*4, 47)

Case5CheckFrame.Action = (function()                        -- modify the check case and stardistLimScale
        return
            function()
                if stardistLimScale ~= 1000000 then 
                    Case1CheckFrame.Text = ""
                    Case2CheckFrame.Text = ""
                    Case3CheckFrame.Text = ""
                    Case4CheckFrame.Text = ""
                    Case5CheckFrame.Text = "o"
                    
                    changeScaleCaseCheck  = true                    
					stardistLimScale = 1000000
                end
            end
        end) ();
